"""Flow control."""

ALLOW = 0
SKIP = 1
HALT = 2


class FlowControl:
    """Control flow of objects in the pipeline."""

    def __init__(self, config):
        """Initialization."""

    def adjust_flow(self, category):
        """Adjust the flow of source control objects."""

        return ALLOW
